from kk_ontology_module import map_data, load_onto, load_data

def test_map():
    onto = load_onto()
    data = load_data()
    map_data(onto, data)
    assert onto.patient1.PatientID[0] == 810037882


from kk_ontology_module import map_data, load_onto, load_data

def test_load_no_col_map():
    onto = load_onto()
    data = load_data()
    data = data.drop(columns=['SITE_ICD10_O2','AGE','SEX','BEHAVIOUR_ICD10_O2'])
    map_data(onto, data)
    assert onto.patient1.PatientID[0] == 810037882


import pandas as pd
import numpy as np
from kk_ontology_module.load_onto import load_onto

def to_pandas(df, onto_class, IDcolname="LINKNUMBER", returns='patient', onto=load_onto()):
    """
    Arguments: 
    df = full dataframe (containing all data)
    onto_class = can be regimen or tumour subclass that you would like to return individuals of
    IDcolname = column name of the PatientID (NHS number, Link number, etc.), TumourID or RegimenID
    onto = full ontology

    Takes all instances of the ontology regimen class and returns the pandas df.
    Currently only supports regimen, tumour and patient subclasses.
    """
    patientIDs = []
    tumourIDs = []
    regimenIDs = []
    
    for instance in onto_class.instances():
        ### TODO: Add functionality for DRUG instances

        if (onto.search(subclass_of = onto.Regimen).count(onto_class)):
        # if onto_class in onto.Regimen.subclasses():
            try:
                patientIDs.append(instance.treats[0].belongs_to_patient[0].PatientID[0])
                tumourIDs.append(instance.treats[0].TumourID[0])
                regimenIDs.append(instance.RegimenID[0])
            except:
                print("Empty instance")
        elif (onto.search(subclass_of = onto.Tumour).count(onto_class)):
        # elif onto_class in onto.Tumour.subclasses():
            try:
                patientIDs.append(instance.belongs_to_patient[0].PatientID[0])
                tumourIDs.append(instance.TumourID[0])
                regimenIDs.append(instance.treated_by[0].RegimenID[0])
            except:
                print("Empty instance")
        elif (onto.search(subclass_of = onto.Patient).count(onto_class)):
        # elif onto_class in onto.Patient.subclasses():
            try:
                patientIDs.append(instance.PatientID[0])
                tumourIDs.append(instance.has_tumour[0].TumourID)
                regimenIDs.append(instance.has_tumour[0].treated_by[0].RegimenID[0])
            except:
                print("Empty instance")
        elif (onto.search(subclass_of = onto.Drug).count(onto_class)):
            try:
                patientIDs.append(instance.part_of_regimen[0].treats[0].belongs_to_patient[0].PatientID[0])
                tumourIDs.append(instance.part_of_regimen[0].treats[0].TumourID)
                regimenIDs.append(instance.part_of_regimen[0].RegimenID[0])
            except:
                print("Empty instance")
        else:
            print("Class is not supported. Currently supported subclasses are of type Drug, Regimen, Tumour, and Patient.")
            return

    # print(patientIDs)
    # returns the dataframe where patientID is included
    if returns=='patient':
        return df[df[IDcolname].isin(patientIDs)]
    elif returns=='regimen':
        return df[df[IDcolname].isin(regimenIDs)]
    elif returns=='tumour':
        return df[df[IDcolname].isin(tumourIDs)]
    else:
        print("Error: returns must be set to patient, regimen or tumour (as a string)")
        return

default_breast_cancer_codes = ['C50', 'C500','C501','C502','C503','C504','C505','C506','C507','C508','C509']
def has_icd10_code(df, icd10colname='PRIMARY_DIAGNOSIS', codes=default_breast_cancer_codes):
    """
    df = pandas dataframe with data
    icd10colname = column name of the ICD10 codes
    codes = python list of codes that you would like to filter by
    """
    return df[df[icd10colname].isin(codes)]

def has_morphology_code(df, morphcolname='MORPH_ICD10_O2', codes=[8090]):
    """
    df = pandas dataframe with data
    morphcolname = column name of the morphology code
    codes = python list of codes that you would like to filter by
    """
    return df[df[morphcolname].isin(codes)]


# TODO: Add numpy support


import pandas as pd
import pkg_resources

TEST_DATA = 'extras/m2dummyB_small.csv'

def load_data(filename=0, mapped_regimen='MAPPED_REGIMEN'):
    stream = pkg_resources.resource_stream(__name__, TEST_DATA)
    if filename:
        # If the user wants to upload their own ontology
        try:
            df = pd.read_csv(filename)
        except(FileNotFoundError):
            print("FileNotFoundError: Please double check the path and name of your data file.")
            return
    else:
        # Using the default ontology shipped with the package
        try:
            df = pd.read_csv(stream)
        except(FileNotFoundError):
            print("Error loading the test_data file shipped with this package.")
            return

    df = df.astype({mapped_regimen: str})
    return df



from kk_ontology_module.load_data import load_data
from kk_ontology_module.load_onto import load_onto
from kk_ontology_module.map_data import map_data
from kk_ontology_module.datamanip import to_pandas, has_icd10_code, has_morphology_code
from kk_ontology_module.reason import CancerOntology, test_data


from owlready2 import *
from contextlib import suppress # for no column name


class Instances():
    patients = []
    tumours = []
    drugs = []
    regimens = []

i = Instances()

class WrongTypeError(Exception):
    """Error when the wrong type is used (i.e. string instead of int, etc.)"""
    pass

def map_data(onto, data_to_map,
             patient_id_col='LINKNUMBER', 
             tumour_id_col='MERGED_TUMOUR_ID',
             tumour_icd10_col = 'SITE_ICD10_O2',
             tumour_behaviour_col='BEHAVIOUR_ICD10_O2',
             regimen_id_col = 'MERGED_REGIMEN_ID',
             patient_age='AGE',
             vital_status='NEWVITALSTATUS',
             patient_primary_diagnosis='PRIMARY_DIAGNOSIS',
             sex_col='SEX',
             diagnosis_date_col='DIAGNOSISDATEBEST',
             regimen_string_col='MAPPED_REGIMEN'):
    ## --------------------- HELPER FUNCTIONS ---------------------
    def fec(instances, regimen):
        '''
        For handling the case where "FEC" is included in the regimen
        '''
        instances.drugs.append(onto.Drug(has_drug_reference=[onto.Fluorouracil5REF], part_of_regimen=[regimen]))
        instances.drugs.append(onto.Drug(has_drug_reference=[onto.EpirubicinREF], part_of_regimen=[regimen]))
        instances.drugs.append(onto.Drug(has_drug_reference=[onto.CyclophosphamideREF], part_of_regimen=[regimen]))

    def tch(instances, regimen):
        '''
        For handling the case where "FEC" is included in the regimen
        '''
        instances.drugs.append(onto.Drug(has_drug_reference=[onto.CarboplatinREF], part_of_regimen=[regimen]))
        instances.drugs.append(onto.Drug(has_drug_reference=[onto.DocetaxelREF], part_of_regimen=[regimen]))
        instances.drugs.append(onto.Drug(has_drug_reference=[onto.TrastuzumabREF], part_of_regimen=[regimen]))

    def tac(instances, regimen):
        '''
        For handling the case where "FEC" is included in the regimen
        '''
        instances.drugs.append(onto.Drug(has_drug_reference=[onto.CyclophosphamideREF], part_of_regimen=[regimen]))
        instances.drugs.append(onto.Drug(has_drug_reference=[onto.DocetaxelREF], part_of_regimen=[regimen]))
        instances.drugs.append(onto.Drug(has_drug_reference=[onto.DoxorubicinREF], part_of_regimen=[regimen]))
    
    def create_drug_instances(string, regimen):
        '''
        Creating drug instances beased on the string (string describes the regimen, usually has drug names)
        TODO: add more drug compatibility.
        '''
        if type(string) != str:
            print("Error: please make sure that the 'MAPPED REGIMEN' column is correct, and is in a string form.")
            raise(WrongTypeError)

        if "capecitabine" in string.lower():
            i.drugs.append(onto.Drug(has_drug_reference=[onto.CapecitabineREF], part_of_regimen=[regimen]))
        if "carboplatin" in string.lower():
            i.drugs.append(onto.Drug(has_drug_reference=[onto.CarboplatinREF], part_of_regimen=[regimen]))
        if "cyclophosphamide" in string.lower():
            i.drugs.append(onto.Drug(has_drug_reference=[onto.CyclophosphamideREF], part_of_regimen=[regimen]))
        if "docetaxel" in string.lower():
            i.drugs.append(onto.Drug(has_drug_reference=[onto.DocetaxelREF], part_of_regimen=[regimen]))
        if "epirubicin" in string.lower():
            i.drugs.append(onto.Drug(has_drug_reference=[onto.EpirubicinREF], part_of_regimen=[regimen]))
        if "fluorouracil" in string.lower():
            i.drugs.append(onto.Drug(has_drug_reference=[onto.Fluorouracil5REF], part_of_regimen=[regimen]))
        if "paclitaxel" in string.lower():
            i.drugs.append(onto.Drug(has_drug_reference=[onto.PaclitaxelREF], part_of_regimen=[regimen]))
        if "pertuzumab" in string.lower():
            i.drugs.append(onto.Drug(has_drug_reference=[onto.PertuzumabREF], part_of_regimen=[regimen]))
        if "trastuzumab" in string.lower():
            i.drugs.append(onto.Drug(has_drug_reference=[onto.TrastuzumabREF], part_of_regimen=[regimen]))
        if "capecitabine" in string.lower():
            i.drugs.append(onto.Drug(has_drug_reference=[onto.CapecitabineREF], part_of_regimen=[regimen]))
        if "cisplatin" in string.lower():
            i.drugs.append(onto.Drug(has_drug_reference=[onto.CisplatinREF], part_of_regimen=[regimen]))
        if "oxaliplatin" in string.lower():
            i.drugs.append(onto.Drug(has_drug_reference=[onto.OxaliplatinREF], part_of_regimen=[regimen]))
        if "rituximab" in string.lower():
            i.drugs.append(onto.Drug(has_drug_reference=[onto.RituximabREF], part_of_regimen=[regimen]))
        if "doxorubicin" in string.lower():
            i.drugs.append(onto.Drug(has_drug_reference=[onto.DoxorubicinREF], part_of_regimen=[regimen]))
        if "vincristine" in string.lower():
            i.drugs.append(onto.Drug(has_drug_reference=[onto.VincristineREF], part_of_regimen=[regimen]))
        if "cytarabine" in string.lower():
            i.drugs.append(onto.Drug(has_drug_reference=[onto.CytarabineREF], part_of_regimen=[regimen]))
        if "temozolomide" in string.lower():
            i.drugs.append(onto.Drug(has_drug_reference=[onto.TemozolomideREF], part_of_regimen=[regimen]))
        if "hydroxycarbamide" in string.lower():
            i.drugs.append(onto.Drug(has_drug_reference=[onto.HydroxycarbamideREF], part_of_regimen=[regimen]))
        
        if "FEC" in string:
            fec(i, regimen)
        if "TCH" in string:
            tch(i, regimen)
        if "TAC" in string:
            tac(i, regimen)
    
    def onto_behaviour_code(onto, code):
        '''
        Converts an integer (behaviour code) into the BehaviourCodeREF instance in ontology
        '''
        if code == 0:
            return onto.BehaviourCode0REF
        if code == 1:
            return onto.BehaviourCode1REF
        if code == 2:
            return onto.BehaviourCode2REF
        if code == 3:
            return onto.BehaviourCode3REF
        if code == 5:
            return onto.BehaviourCode5REF
        if code == 6:
            return onto.BehaviourCode6REF
        if code == 9:
            return onto.BehaviourCode9REF
        return 0 # error case, but not a big deal to raise an error
    
    def tumour_icd10_code(onto, code):
        '''
        Converts string of ICD10 code to Tumour ICD10 code REF instance in ontology.
        '''
        code1 = code[0] ## First letter
        code2 = code[1:3] ## First 2 numbers
        if code1 == "C":
            if 0 <= int(code2) <= 14:
                return onto.C00_C14_REF
            if 15 <= int(code2) <= 26:
                return onto.C15_C26_REF
            if 30 <= int(code2) <= 39:
                return onto.C30_C39_REF
            if 40 <= int(code2) <= 41:
                return onto.C40_C41_REF
            if 43 <= int(code2) <= 44:
                return onto.C43_C44_REF
            if 45 <= int(code2) <= 49:
                return onto.C45_C49_REF
            if int(code2) == 50:
                return onto.C50_C50_REF
            if 51 <= int(code2) <= 58:
                return onto.C51_C58_REF
            if 60 <= int(code2) <= 63:
                return onto.C60_C63_REF
            if 64 <= int(code2) <= 68:
                return onto.C64_C68_REF
            if int(code2) == 71:
                return onto.C71_REF 
            if 69 <= int(code2) <= 72:
                return onto.C69_C72_REF
            if 73 <= int(code2) <= 75:
                return onto.C73_C75_REF
            if 76 <= int(code2) <= 80:
                return onto.C76_C80_REF
            if 81 <= int(code2) <= 96:
                return onto.C81_C96_REF
            if code2 == "7A":
                return onto.C7A_C7A_REF
            if code2 == "7B":
                return onto.C7B_C7B_REF
        if code1 == "D":
            if 0 <= int(code2) <= 9:
                return onto.D00_D09_REF
            if 10 <= int(code2) <= 36:
                return onto.D10_D36_REF
            if 37 <= int(code2) <= 48:
                return onto.D37_D48_REF
            if int(code2) == 49:
                return onto.D49_D49_REF
            if code2 == "3A":
                return onto.D3A_D3A_REF
        return 0 # error case

    
    ## --------------------- MAIN FUNCTION ---------------------
    def create_instances(data):
        '''
        Mapping instances of data to individuals in the ontology.
        Mandatory parameters are PatientID, TumourID, RegimenID, RegimenStringColumn, VitalStatus, PrimaryDiagnosis
        Optional parameters are Age, Sex, TumourBehaviourCode, TumourICD10Column
        '''

        for index, row in data.iterrows():
            # Create a new patient instance, but checks if patient has been created before
            patient_search = onto.search(PatientID = str(row[patient_id_col])+"*")
            if not patient_search:
                today = datetime.date.today()
                yearBorn = datetime.date(today.year-row[patient_age],1,1).year if patient_age in data.columns else 0
                thisPatient = onto.Patient(PatientID = [row[patient_id_col]], ## equivalent of NHS number 
                                            DateOfBirth = [yearBorn],
                                            VitalStatus = [row[vital_status]],
                                            PrimaryDiagnosis = [row[patient_primary_diagnosis]], ## patient primary tumour icd10
                                            Sex = [row[sex_col]] if sex_col in data.columns else [0]
                                            )
            else:
                thisPatient = patient_search[0]
            i.patients.append(thisPatient)

            # Create a new tumour instance, also checks if tumour has been created before
            tumour_search = onto.search(TumourID = str(row[tumour_id_col])+"*")
            if not tumour_search:
                thisTumour = onto.Tumour(TumourID = [row[tumour_id_col]],
                                            DiagnosisDate = [row[diagnosis_date_col]],
                                            ICD10_Code = [row[tumour_icd10_col]] if tumour_icd10_col in data.columns else [0], ## tumour icd10
                                            has_behaviour_code = [onto_behaviour_code(onto, row[tumour_behaviour_col])] if tumour_behaviour_col in data.columns else [0], # behaviour code
                                            belongs_to_patient = [thisPatient],
                                            has_tumour_reference = [tumour_icd10_code(onto, row[tumour_icd10_col])]
                                            )
            else:
                thisTumour = tumour_search[0]
            i.tumours.append(thisTumour)

            # Create a new regimen instance, doesn't need to check if been created before, as new regimen is new row.
            thisRegimen = onto.Regimen(RegimenID = [row[regimen_id_col]], 
                                        treats = [thisTumour]
                                        )
            i.regimens.append(thisRegimen)

            # Create new drug instances in a for loop using names, then all drugs part of this regimen
            create_drug_instances(row[regimen_string_col], thisRegimen)
    
    # ----------- CREATING INSTANCES FUNCTION CALLED -----------------
    with suppress(KeyError):
        create_instances(data_to_map)


## Defining the load function
from owlready2 import *
import pkg_resources

TEST_ONTO = "extras/CancerOntology.owl"

def load_onto(filename=0):
    stream = pkg_resources.resource_filename(__name__, TEST_ONTO)
    if filename:
        try:
            o = get_ontology(filename).load()
        except(FileNotFoundError):
            print("FileNotFoundError: Please double check the path and name of your ontology file.")
            return
    else:
        try:
            o = get_ontology(stream).load()
        except(FileNotFoundError):
            print("Error loading the ontology file shipped with this package.")
            return
    
    return o


from owlready2 import *
from kk_ontology_module.load_data import load_data, TEST_DATA
from kk_ontology_module.load_onto import load_onto, TEST_ONTO
from kk_ontology_module.map_data import map_data

test_onto = load_onto() # default ontology
test_data = load_data() # default data

## Defining class cancer ontology
class CancerOntology:
    def __init__(self, onto=test_onto):
        self.onto = onto
        self.default_world = World()
        self.world = default_world

        # setting the default column names: these should be able to be edited
        self.patient_id_col = 'LINKNUMBER'
        self.tumour_id_col = 'MERGED_TUMOUR_ID'
        self.tumour_icd10_col = 'SITE_ICD10_O2'
        self.tumour_behaviour_col = 'BEHAVIOUR_ICD10_O2'
        self.regimen_id_col = 'MERGED_REGIMEN_ID'
        self.patient_age = 'AGE'
        self.vital_status = 'NEWVITALSTATUS'
        self.patient_primary_diagnosis = 'PRIMARY_DIAGNOSIS'
        self.sex_col = 'SEX'
        self.diagnosis_date_col = 'DIAGNOSISDATEBEST'
        self.regimen_string_col = 'MAPPED_REGIMEN'

    def reason(self):
        """
        Reasons with HermiT. From the Owlready2 Package.
        """
        with self.onto: sync_reasoner(self.world)

    def reload(self):
        """
        Function to reload the ontology
        TODO: Unfortunately this doesn't work...
        """
        print("Previous ontology deleted.")
        self.world = default_world
        print("New ontology loaded.")
    
    def define_data(self, data):
        
        self.data = data
        map_data(self.onto, data,
                 patient_id_col = self.patient_id_col, 
                 tumour_id_col = self.tumour_id_col,
                 tumour_icd10_col = self.tumour_icd10_col,
                 tumour_behaviour_col = self.tumour_behaviour_col,
                 regimen_id_col = self.regimen_id_col,
                 patient_age = self.patient_age,
                 vital_status = self.vital_status,
                 patient_primary_diagnosis = self.patient_primary_diagnosis,
                 sex_col = self.sex_col,
                 diagnosis_date_col = self.diagnosis_date_col,
                 regimen_string_col = self.regimen_string_col)
    
    def set_gliocova_column_names(self):
        self.patient_id_col = 'PSEUDO_PATIENTID'
        self.tumour_id_col = 'PSEUDO_TUMOURID'
        self.regimen_id_col = 'REGIMEN_NUMBER'
        self.vital_status = 'VITALSTATUS'
        self.diagnosis_date_col = 'DIAGNOSISYEAR'
        self.regimen_string_col = 'BENCHMARK_GROUP'
        return


# def test1():
#     o1 = CancerOntology()
#     o1.onto.Drug("test_drug", Dose = [100], has_drug_reference = [o1.onto.CyclophosphamideREF])
#     print(o1.onto.test_drug)
#     o1.reload(test_onto)
#     print(o1.onto.test_drug)

# def test2():
#     o2 = CancerOntology()
#     o2.define_data(test_data)
#     # print(o2.onto.Regimen.instances())

# def test3():
#     o3 = CancerOntology()
#     o3.define_data(test_data)
#     o3.reason()
#     print(o3.onto.DocetaxelDrug.instances())

# def test4():
#     # data = load_data("extras/m2dummyB_med.csv")
#     o4 = CancerOntology()
#     o4.define_data(test_data)
#     o4.reason()
#     print(to_pandas(test_data, o4.onto.TaxaneContainingRegimen))
#     # print(to_pandas(test_data, o4.onto.TaxaneContainingRegimen, IDcolname='MERGED_REGIMEN_ID', returns='regimen'))
#     # print(to_pandas(test_data, o4.onto.TaxaneContainingRegimen, IDcolname='MERGED_TUMOUR_ID', returns='tumour'))

# def test5():
#     o5 = CancerOntology()
#     o5.define_data(test_data)
#     o5.reason()
#     print(has_morphology_code(test_data))

# def test6():
#     o6 = CancerOntology()
#     o6data = load_data('extras/m2dummy_med.csv')
#     o6.define_data(o6data)
#     # o6.define_data
#     o6.reason()
#     # print(o6.onto.search(subclass_of = o6.onto.Tumour))
#     print(to_pandas(o6data, o6.onto.PlatinumBasedRegimen, IDcolname='MERGED_REGIMEN_ID', returns='regimen'))

# def test7():
#     o7 = CancerOntology()
#     o7.define_data(test_data)
#     o7.reason()
#     # print(o6.onto.search(subclass_of = o6.onto.Tumour))
#     print(to_pandas(test_data, o7.onto.Tumour_C43_C44))

# test1()

# o1.reason()
# print(o1.onto.E1.has_drug_reference)
# print(o1.onto.EpirubicinDrug.instances())


from kk_ontology_module import map_data, load_onto, load_data, to_pandas

def test_to_pandas():
    onto = load_onto()
    data = load_data()
    map_data(onto, data)
    df = to_pandas(data, onto.Patient)
    assert df.iloc[0][1] == 10037882


from setuptools import setup, find_packages

VERSION = '1.0.22' 
DESCRIPTION = 'Ontology-based tool for cancer data analysis'
LONG_DESCRIPTION = 'A packaged set of functions for ontology-based reasoning of cancer patients'

# Setting up
setup(
       # the name must match the folder name 'verysimplemodule'
        name="kk_ontology_module", 
        version=VERSION,
        author="Kaixuan Khoo",
        author_email="<kaixuan.khoo.98@gmail.com>",
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        packages=find_packages(),
        package_data={'kk_ontology_module': ['extras/*']},
        install_requires=["cython", "wheel", "owlready2", "pandas"], # add any additional packages that 
        # needs to be installed along with your package. Eg: 'caer'
        
        keywords=['python', 'ontology', 'cancer'],
        classifiers= [
            "Development Status :: 3 - Alpha",
            "Intended Audience :: Education",
            "Programming Language :: Python :: 2",
            "Programming Language :: Python :: 3",
            "Operating System :: MacOS :: MacOS X",
            "Operating System :: Microsoft :: Windows",
        ]
)


from kk_ontology_module import CancerOntology, test_data
import pandas as pd

def test_gliocova():
    o = CancerOntology()
    df = test_data.rename(columns={'LINKNUMBER': 'PSEUDO_PATIENTID', 'MERGED_TUMOUR_ID': 'PSEUDO_TUMOURID', 
        'NEWVITALSTATUS':'VITALSTATUS', 'MERGED_REGIMEN_ID':'REGIMEN_NUMBER', 'DIAGNOSISDATEBEST':'DIAGNOSISYEAR'})
    o.set_gliocova_column_names()
    o.define_data(df)
    o.reason()
    assert o.onto.patient2.PatientID == [810089744]


from kk_ontology_module import map_data, load_onto, load_data, to_pandas, CancerOntology, test_data

def test_reason():
    o = CancerOntology()
    o.define_data(test_data)
    o.reason()
    df = to_pandas(test_data, o.onto.TaxaneContainingRegimen)

    assert df.iloc[0][1] == 10037882